#ifndef __PACKET_HEADER_INCLUDED__
#define __PACKET_HEADER_INCLUDED__

#include <g_globals.h>
#include <capdev.h>

#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <netinet/igmp.h>
#include <netinet/ip.h>
#include <netinet/ip6.h>
#include <netinet/ether.h>
#include <netinet/if_ether.h>

typedef struct ether_hdr {
	uchar	d_addr[6];
	uchar	s_addr[6];
	ushort	type;
}ether_hdr_t;

#define PORT ushort
#define CAP_SNAPLEN 4096

typedef enum network_layer{
	LAYER_LINK,
	LAYER_NET,
	LAYER_TRANS,
	LAYER_APP,
	LAYER_MAX
}network_layer_t;

typedef struct layer {
	uint id;
	char *name;
	union {
		void			*raw;
		struct tcphdr  		*tcp;
		struct udphdr		*udp;
		struct icmphdr		*icmp;
		struct igmphdr		*igmp;
		struct iphdr		*ip;
		struct ipv6hdr		*ipv6;
		struct ether_arp	*arp;
		struct ether_hdr	*mac;
	}h;
}layer_t, *p_layer;

typedef struct packet {
	time_t		time;	/* Timestamp */
	struct tm	*ltime;	/* Localised timestamp */
	
	uint		caplen;	/* Length of present data */
	uint		len;	/* Real length of data on the wire */

	p_capdev	capdev; /* The capture device that cought it */
	
	PORT		sport, dport; /* Source and dest ports */

	layer_t         layer[LAYER_MAX]; /* Network layers */
	uchar		data[CAP_SNAPLEN]; /* Full packet data */
}packet_t, *p_packet;

#endif //__PACKET_HEADER_INCLUDED__
