#include <ip.h>

proc_print_out print_out;
proc_print_raw print_raw;
proc_demux_add_handler add_demux;

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);

	print_out=in->print_out;
	print_raw=in->print_raw;
	add_demux=in->add_demux;

	out->name="IP";
	out->desc="TCP/IP networking protocols suite";
	out->ver_major=1;
	out->ver_minor=0;
	
	if ( !(add_demux(LAYER_NET, demux_callback)) ) {
		return PLUGIN_ERR_FAIL;
	}

	if ( !(add_demux(LAYER_TRANS, demux_callback2)) ) {
		return PLUGIN_ERR_FAIL;
	}
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}

/* This is our actual demultiplex handler */

#define NPROTO_IP	0x0800

/* IP itself */
demux_pret_t demux_callback(p_packet p)
{
	if ( !p->layer[LAYER_LINK].h.raw ) return DEMUX_NULL;
	if ( !p->layer[LAYER_NET].h.raw ) return DEMUX_NULL;
	
	if ( p->layer[LAYER_NET].id == NPROTO_IP ) {
		p->layer[LAYER_NET].name = "IPv4";
		p->layer[LAYER_TRANS].id = p->layer[LAYER_NET].h.ip->protocol;
		p->layer[LAYER_TRANS].h.raw = p->layer[LAYER_NET].h.raw  + 
			p->layer[LAYER_NET].h.ip->ihl*4;
		return DEMUX_SKIP;
	}
	
	return DEMUX_NULL;
}

/* All our adjuncts */
demux_pret_t demux_callback2(p_packet p)
{
	if ( !p->layer[LAYER_LINK].h.raw ) return DEMUX_NULL;
	if ( !p->layer[LAYER_NET].h.raw ) return DEMUX_NULL;
	if ( !p->layer[LAYER_TRANS].h.raw ) return DEMUX_NULL;
	
	/* These transport layers are for IP only*/
	if ( p->layer[LAYER_NET].id != NPROTO_IP ) return DEMUX_NULL;

	switch ( p->layer[LAYER_TRANS].id ) {
		case IPPROTO_TCP:
			p->layer[LAYER_TRANS].name = "TCP";
			p->sport = ntohs(p->layer[LAYER_TRANS].h.tcp->source);
			p->dport = ntohs(p->layer[LAYER_TRANS].h.tcp->dest);
			p->layer[LAYER_APP].h.raw = p->layer[LAYER_TRANS].h.raw + 
				p->layer[LAYER_TRANS].h.tcp->doff;
			break;
		
		case IPPROTO_UDP:
			p->layer[LAYER_TRANS].name = "UDP";
			p->sport = ntohs(p->layer[LAYER_TRANS].h.udp->source);
			p->dport = ntohs(p->layer[LAYER_TRANS].h.udp->dest);
			p->layer[LAYER_APP].h.raw = p->layer[LAYER_TRANS].h.raw +
				sizeof(struct udphdr);
			break;

		case IPPROTO_ICMP:
			p->layer[LAYER_TRANS].name = "ICMP";
			p->sport = p->layer[LAYER_TRANS].h.icmp->type;
			p->dport = p->layer[LAYER_TRANS].h.icmp->code;
			p->layer[LAYER_APP].h.raw = p->layer[LAYER_TRANS].h.raw +
				sizeof(struct icmphdr);
			break;

		case IPPROTO_IGMP:
			p->layer[LAYER_TRANS].name = "IGMP";
			p->layer[LAYER_APP].h.raw = p->layer[LAYER_TRANS].h.raw +
				sizeof(struct igmphdr);
			break;
			
		default:
			return DEMUX_NULL;
			
	}
	
	p->layer[LAYER_APP].id = 0;
	p->layer[LAYER_APP].name = NULL;
	return DEMUX_SKIP;
}
